****************************************
*                                      *
*      PROJECT: Show EXIF information  *
*         FILE: Exif.prg               *
*   Programmer: J.A. Diego Kerejeta    *
*      Created: 27-06-2014             *
*      Version: 0.01                   *
*                                      *
****************************************

#include "Appevent.ch"
#include "Common.ch"
#include "Xbp.ch"
#include 'activeX.ch'

#define UndefinedImagePropertyType	           1000 // The value of the image property is undefined and returned as a Byte.
#define ByteImagePropertyType	                   1001 // The value of the image property is a Byte.
#define StringImagePropertyType	                   1002 // The value of the image property is a String.
#define UnsignedIntegerImagePropertyType           1003 // The value of the image property is returned as a non-negative Integer.
#define LongImagePropertyType                      1004 // The value of the image property is a Long.
#define UnsignedLongImagePropertyType	           1005 // The value of the image property is returned as a non-negative Long.
#define RationalImagePropertyType	           1006 // The value of the image property is returned as a Rational Object.
#define UnsignedRationalImagePropertyType          1007 // The value of the image property is returned as an unsigned Rational Object.
#define VectorOfUndefinedImagePropertyType         1100 // The value of the image property is returned as a Vector object containing Byte elements.
#define VectorOfBytesImagePropertyType	           1101 // The value of the image property is a Vector object containing Byte elements.
#define VectorOfUnsignedIntegersImagePropertyType  1102 // The value of the image property is returned as a Vector object containing Integer elements.
#define VectorOfLongsImagePropertyType	           1103 // The value of the image property is a Vector object containing Long elements.
#define VectorOfUnsignedLongsImagePropertyType	   1104 // The value of the image property is returned as a Vector object containing Long elements.
#define VectorOfRationalsImagePropertyType	   1105 // The value of the image property is returned as a Vector object containing Rational object elements.
#define VectorOfUnsignedRationalsImagePropertyType 1106 // The value of the image property is returned as a Vector object containing unsigned Rational object elements.




*************************
PROCEDURE AppSys ; RETURN
PROCEDURE DbeSys ; RETURN
*************************

**************
PROCEDURE Main
**************
LOCAL oDlg, oLB, nEvent, mp1, mp2, oXbp

SET CHARSET TO ANSI

oDlg := XbpDialog():new( , , { 0, 0 }, { 200, 200 }, , .F. )
   oDlg:taskList:= .T.
   oDlg:title   := "Test"
   oDlg:create()
   oDlg:setFrameState( XBPDLG_FRAMESTAT_MAXIMIZED )

   oLB:= XbpListBox():new():create( oDlg:drawingArea,,, oDlg:drawingArea:currentSize() )

   oDlg:show()

   SetAppWindow( oDlg )
   SetAppFocus( oLB )

   ExifInfo( CurDrive() + ':\' + CurDir() + '\gps_data.JPG', oLB )

   oLB:setData( 1 )

   DO WHILE nEvent <> xbeP_Close
      nEvent:= AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO
RETURN

********************************
PROCEDURE ExifInfo( cFile, oLB )
********************************
LOCAL oImg, nCont, oProperties, oProp, cProp, nLen, xValue, cLBItem

IF FExists( cFile )
   oImg:= CreateObject( 'WIA.ImageFile' )

   oImg:loadFile( cFile )

   oProperties:= oImg:properties

   nLen:= oProperties:count

   oLB:lockUpdate( .T. )

   FOR nCont:= 1 TO nLen
      oProp  := oProperties:item( nCont )
      xValue := oProp:value
      cLBItem:= LTrim( StrZero( nCont, 2 ) ) + ', ' + var2char( oProp:name ) + ': '
      cProp  := oProp:name

      IF Valtype( xValue ) = 'O'
         cProp := '*' + cProp + '*'

         IF cProp $ '*GpsLatitude*GpsLongitude*'
            cLBItem += DecodeVofRAngle( xValue  )
         ELSEIF cProp $ '40091*40092*40093*40094*40095*'
            cLBItem += xValue:string
         ELSEIF cProp $ '*ExifCompConfig*ExifCfaPattern*'
            cLBItem += DecodeNumericBytes( xValue, '' )
         ELSEIF cProp $ '*20546*ExifVer*ExifUserComment*ExifFPXVer*'
            cLBItem += DecodeAsciiBytes( xValue )
         ELSEIF cProp == '*GpsGpsTime*'
            cLBItem +=  DecodeVofRTime( xValue ) + ' UTC'
         ELSEIF cProp == '*GpsVer*'
            cLBItem += DecodeNumericBytes( xValue, '.' )
         ELSEIF cProp $ '*37396*BitsPerSample*'
            cLBItem += DecodeNumericBytes( xValue, ', ' )
         ELSEIF cProp == '*ExifExposureTime*'
            cLBItem += Str( ( xValue:numerator / xValue:denominator ), 10, 6 ) + ' second'


// new process model 


         ELSEIF oProp:Type = RationalImagePropertyType .OR. oProp:Type = UnsignedRationalImagePropertyType
           cLBItem += LTrim( Str( xValue:numerator )  ) + '/'  + LTrim( Str( xValue:denominator ) )


         ELSE
            cLBItem += Var2Char( xValue )
         ENDIF

*        cLBItem += ', ' + var2char( oProp:type )

         xValue:destroy()
      ELSEIF cProp == 'GpsImgDirRef'
         IF xValue == ""
            xValue = '*none*'
         ELSEIF Lower( xValue ) == 't'
            xValue:= 'true'
         ELSEIF Lower( xValue ) == 'm'
            xValue:= 'magnetic'
         ENDIF
         cLBItem += xValue
      ELSEIF cProp == 'GpsAltitudeRef'
         cLBItem += IIF( xValue = 0, 'above sea level', 'below sea level' )
      ELSEIF cProp == 'GpsAltitude'
         cLBItem += Var2char( xValue ) + ' meters'
      ELSE
         cLBItem += Var2Char( xValue )
      ENDIF

      oLB:addItem( cLBItem )

      oProp:destroy()
   NEXT
   oLB:lockUpdate( .F. )
   oLB:invalidateRect()

   oProperties:destroy()
   oImg:destroy()
ENDIF
RETURN

**********************************
FUNCTION DecodeVofRAngle( xValue )
**********************************
LOCAL nLen  := xValue:count
LOCAL aStr  := { '', "'", "''" }
LOCAL cValue:= ''
LOCAL nCont, oItem, nVal

FOR nCont:= 1 TO nLen
   oItem  := xValue:item( nCont )
   nVal   := oItem:value
   cValue += LTrim( Str( nVal, 10, IIF( nVal = Int( nVal ), 0, 3 ) ) ) + aStr[ nCont ] + ' '
   oItem:destroy()
NEXT
RETURN cValue

*********************************
FUNCTION DecodeVofRTime( xValue )
*********************************
LOCAL nLen  := xValue:count
LOCAL cValue:= ''
LOCAL nCont, oItem, nVal

FOR nCont:= 1 TO nLen
   oItem  := xValue:item( nCont )
   nVal   := oItem:value
   cValue += LTrim( Str( nVal, 10, IIF( nVal = Int( nVal ), 0, 2 ) ) ) + ':'
   oItem:destroy()
NEXT
RETURN Left( cValue, Len( cValue ) - 1 )

********************************************
FUNCTION DecodeNumericBytes( xValue, cChar )
********************************************
LOCAL nLen  := xValue:count
LOCAL cValue:= ''
LOCAL nCont

FOR nCont:= 1 TO nLen
   cValue += Var2char( xValue:item( nCont ) ) + cChar
NEXT
RETURN IIF( cChar == '', cValue, Left( cValue, Len( cValue ) - Len( cChar ) ) )

*************************************
FUNCTION DecodeAsciiBytes( xValue )
*************************************
LOCAL nLen  := xValue:count
LOCAL cValue:= ''
LOCAL nCont

FOR nCont:= 1 TO nLen
   cValue += Chr( xValue:item( nCont ) )
NEXT
RETURN cValue



